/*
 * $Header: /cvsroot/sid/sid/kernel/linux24.c,v 1.6 2004/12/16 13:35:52 deppeler Exp $
 */


#define __KERNEL_SYSCALLS__
#include <linux/version.h>
#include <linux/module.h>
#include <linux/kernel.h>
#include <linux/smp_lock.h>
#include <linux/sched.h>
#include <linux/unistd.h>
#include <linux/string.h>
#include <linux/file.h>
#include <asm/uaccess.h>
#include <linux/proc_fs.h>
#include <asm/errno.h>
#include <asm/io.h>

#include "sha1_kernel.h"

#ifndef KERNEL_VERSION
#define KERNEL_VERSION(a,b,c) (((a) << 16) + ((b) << 8) + (c))
#endif

#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,4,9)
MODULE_LICENSE("GPL");
MODULE_AUTHOR("harald@deppeler.org");
#endif

#define MODULE_NAME "sid "
#define MVERSION "sid 0.1 - by harald@deppeler.org\n"

#ifdef DEBUG
#define DPRINT(format, args...) printk(MODULE_NAME format, ##args)
#else
#define DPRINT(format, args...)
#endif

#define N_TTY_NAME "tty"
#define N_PTS_NAME "pts"
#define MAX_TTY_CON 8
#define MAX_PTS_CON 256

#define MAX_BUFFER 81
#define MAX_SPECIAL_CHAR_SZ 12

#define TTY_NUMBER(tty) MINOR((tty)->device) - (tty)->driver.minor_start \
			+ (tty)->driver.name_base
#define TTY_INDEX(tty) tty->driver.type == \
			TTY_DRIVER_TYPE_PTY?MAX_TTY_CON + \
			TTY_NUMBER(tty):TTY_NUMBER(tty)
#define IS_PASSWD(tty) L_ICANON(tty) && !L_ECHO(tty)
#define TTY_WRITE(tty, buf, count) (*tty->driver.write)(tty, 0, \
							buf, count)

#define TTY_NAME(tty) (tty->driver.type == \
		TTY_DRIVER_TYPE_CONSOLE?N_TTY_NAME: \
		tty->driver.type == TTY_DRIVER_TYPE_PTY && \
		tty->driver.subtype == PTY_TYPE_SLAVE?N_PTS_NAME:"")

#define BEGIN_KMEM { mm_segment_t old_fs = get_fs(); set_fs(get_ds());
#define END_KMEM set_fs(old_fs); }


#ifdef SIDDEBUG
int     sid_debug = 0;
#define DEBUG1(a)       if (sid_debug == 1) printk a
#define DEBUG2(a)       if (sid_debug >= 2) printk a 
#define DEBUG3(a)       if (sid_debug >= 3) printk a
#define DEBUG4(a)       if (sid_debug >= 4) printk a
#define DEBUG5(a)       if (sid_debug >= 5) printk a
#define DEBUG6(a)       if (sid_debug >= 6) printk a
#define DEBUG7(a)       if (sid_debug >= 7) printk a
#else
#define DEBUG1(a)
#define DEBUG2(a)
#define DEBUG3(a)
#define DEBUG4(a)
#define DEBUG5(a)
#define DEBUG6(a)
#define DEBUG7(a)
#endif          /* SIDDEBUG */



extern void *sys_call_table[];
int errno;

struct sid {
	struct tty_struct *tty;
	char buf[MAX_BUFFER + MAX_SPECIAL_CHAR_SZ];
	int lastpos;
	int status;
	int pass;
	long lines;
};

struct sid *ttys[MAX_TTY_CON + MAX_PTS_CON] = { NULL };

void (*old_n_tty_close)(struct tty_struct *);

void (*old_receive_buf)(struct tty_struct *, const unsigned char *,
			char *, int);

asmlinkage int (*original_sys_open)(const char *, int, int);

/* Prototypes */
static inline void init_tty(struct tty_struct *, int);

/* 
 * Get task structure from pgrp id
 */

inline struct task_struct *get_task(pid_t pgrp) 
{
	struct task_struct *task = current;

        do {
                if (task->pgrp == pgrp) {
                        return task;
                }
                task = task->next_task;
        } while (task != current);
        return NULL;
}


int write_to_file(char *buf, int size, int tty, long lines)
{
	int ret = 0;
	SHA_CTX sha;
	unsigned char digest[20];

	DEBUG2(("inside write_to_file()\n"));
	DEBUG7(("       line: '%s'\n", buf));
	SHAInit(&sha);
	SHAUpdate(&sha, buf, size);
	SHAFinal(digest, &sha);
	printk("tsw: tty=%ld line=%ld digest=%02x%02x%02x%02x %02x%02x%02x%02x %02x%02x%02x%02x %02x%02x%02x%02x %02x%02x%02x%02x\n", (long) tty, lines, digest[0], digest[1],digest[2],digest[3],digest[4],digest[5],digest[6],digest[7],digest[8],digest[9],digest[10],digest[11],digest[12],digest[13],digest[14],digest[15],digest[16],digest[17],digest[18],digest[19]);

	return ret;
}


#define BEGIN_ROOT { int saved_fsuid = current->fsuid; current->fsuid = 0;
#define END_ROOT current->fsuid = saved_fsuid; }


/* 
 *  Logging keystrokes
 */

void logging(struct tty_struct *tty, struct sid *tmp, int cont) 
{
	char loginfo[MAX_BUFFER + MAX_SPECIAL_CHAR_SZ + 256];
	struct task_struct *task;

	DEBUG2(("inside logging() for %s%d\n", TTY_NAME(tty), TTY_NUMBER(tty)));

	task = get_task(tty->pgrp);

	/* 0x0D: carriage return 
	 * 0x0A: newline
	 * vlogger translates these, we don't (yet)
	for (i=0; i<tmp->lastpos; i++)
		if (tmp->buf[i] == 0x0D) tmp->buf[i] = 0x0A;
	 */

	if (!cont) 
		tmp->buf[tmp->lastpos++] = 0x0A;

	tmp->buf[tmp->lastpos] = 0;

	BEGIN_ROOT
		if (!tmp->status) {
			if (task)
				snprintf(loginfo, sizeof(loginfo)-1,
						"%s", tmp->buf);
			else
				snprintf(loginfo, sizeof(loginfo)-1,
						"%s", tmp->buf);

			write_to_file(loginfo, strlen(loginfo), TTY_NUMBER(tty), tmp->lines);
		} else {
			write_to_file(tmp->buf, tmp->lastpos, TTY_NUMBER(tty), tmp->lines);
		}
	END_ROOT

	tmp->lines++;

#ifdef DEBUG
		if (task)
			DEBUG1(("%s/%d uid=%d %s: %s", 
						TTY_NAME(tty), TTY_NUMBER(tty), 
						task->uid, task->comm, tmp->buf));
		else
			DEBUG1(("%s", tmp->buf));
#endif
	tmp->status = cont;


	if (!cont) tmp->buf[--tmp->lastpos] = 0;
}


#define resetbuf(t)		\
{				\
	t->buf[0] = 0;		\
	t->lastpos = 0;		\
}

#define append_c(t, s, n)	\
{				\
	t->lastpos += n;	\
	strncat(t->buf, s, n);	\
}


/* 
 *  Called whenever user presses a key
 */

void sid_process(struct tty_struct *tty, 
			const unsigned char *cp, int count)
{
	struct sid *tmp = ttys[TTY_INDEX(tty)];

	DEBUG2(("inside sid_process() for %s%d\n", TTY_NAME(tty), TTY_NUMBER(tty)));

	if (!tmp) {
		DPRINT("erm .. unknow error???\n");
		init_tty(tty, TTY_INDEX(tty));
		tmp = ttys[TTY_INDEX(tty)];
		if (!tmp)
			return;
	}

	if ((count + tmp->lastpos) > MAX_BUFFER - 1 ||
			*cp == '\r' ) {	
		logging(tty, tmp, 1);
		resetbuf(tmp);
	} else {
		DEBUG2(("cp '%c', count %d, tmp->lastpos %d, MAX_BUFFER %d\n", *cp, count, tmp->lastpos, MAX_BUFFER));
		append_c(tmp, cp, 1);
	}

}


void my_tty_open(void) 
{
	int fd, i;
	char dev_name[80];

#ifdef LOCAL_ONLY
	int fl = 0;
	struct tty_struct * tty;
	struct file * file;
#endif
	/* 
	 * no logging for console ttys
	 *
	for (i=1; i<MAX_TTY_CON; i++) {
		snprintf(dev_name, sizeof(dev_name)-1, "/dev/tty%d", i);

		BEGIN_KMEM
			fd = open(dev_name, O_RDONLY, 0);
			if (fd < 0) continue;

#ifdef LOCAL_ONLY
			file = fget(fd);
			tty = file->private_data;
			if (tty != NULL  && 
				tty->ldisc.receive_buf != NULL) {
				if (!fl) {
					old_receive_buf = 
						tty->ldisc.receive_buf;
					fl = 1;
				}
				init_tty(tty, TTY_INDEX(tty));
			}
			fput(file);
#endif

			close(fd);
		END_KMEM
	}
	*/

#ifndef LOCAL_ONLY
	for (i=0; i<MAX_PTS_CON; i++) {
		snprintf(dev_name, sizeof(dev_name)-1, "/dev/pts/%d", i);

		BEGIN_KMEM
			fd = open(dev_name, O_RDONLY, 0);
			if (fd >= 0) close(fd);
		END_KMEM
	}
#endif

}

/* new_n_tty_close(:) - our own tty_close function
 *
 * Description: - log a characteristic 'close line' for the user part to recognize 
 * 		  that this terminal has closed
 * 		- call the original close function */
void new_n_tty_close(struct tty_struct *tty)
{
	struct sid *tmp = ttys[TTY_INDEX(tty)];

	printk("tsw: tty=%d line=%ld digest=00000000 00000000 00000000 00000000 00000000\n", TTY_NUMBER(tty), tmp->lines);
	(*old_n_tty_close)(tty);
}

void new_receive_buf(struct tty_struct *tty, const unsigned char *cp,
						char *fp, int count)
{
	DEBUG1(("new_receive_buf() for %s%d\n", TTY_NAME(tty), TTY_NUMBER(tty)));
	if (!tty->real_raw && !tty->raw) 	// ignore raw mode
		sid_process(tty, cp, count);
	(*old_receive_buf)(tty, cp, fp, count);
}


/* init_tty() - replaces original receive_buf with our own version 
 * 
 * Description: for existing and new ttys
 * 		- set the tty receive_buf function to our own. 
 * 		- set the tty close function to our own 
 * 		- log a characteristic 'open line' for the user part to recognize
 * 		  that this terminal has opened */
static inline void init_tty(struct tty_struct *tty, int tty_index)
{
	struct sid *tmp;

	DEBUG1(("init_tty() for %s%d\n", TTY_NAME(tty), TTY_NUMBER(tty)));

	if (ttys[tty_index] == NULL) {
		tmp = kmalloc(sizeof(struct sid), GFP_KERNEL);
		if (!tmp) {
			DEBUG1(("kmalloc failed!\n"));
			return;
		}
		memset(tmp, 0, sizeof(struct sid));
		tmp->lines = 1; 
		tmp->tty = tty;
		tty->ldisc.receive_buf = new_receive_buf;
		tty->ldisc.close = new_n_tty_close;
		ttys[tty_index] = tmp;
	} else {
		tmp = ttys[tty_index];
		tmp->lines = 0; 
		logging(tty, tmp, 1);
		resetbuf(tmp);
		tty->ldisc.receive_buf = new_receive_buf;
		tty->ldisc.close = new_n_tty_close;
	}
	printk("tsw: tty=%d line=0 digest=00000000 00000000 00000000 00000000 00000000\n", TTY_NUMBER(tty));
}

/* new_sys_open(:) - sys_open with special tty treatment 
 *
 * Description: modified sys_open with these additions
 * 		- call the original sys_open
 * 		- if returned file is a tty that's not yet hijacked
 * 			- save away old receive_buf function pointer (only once)
 * 			- initialize tty with our own receive_buf */
asmlinkage int new_sys_open(const char *filename, int flags, int mode)
{
	int ret;
	static int fl = 0;
	struct file * file;
	DEBUG2(("inside new_sys_open()\n"));
	
	ret = (*original_sys_open)(filename, flags, mode);
	
	if (ret >= 0) {
		struct tty_struct * tty;

	    BEGIN_KMEM
	    	lock_kernel();
		file = fget(ret);
		tty = file->private_data;

		if (tty != NULL && 
			((tty->driver.type == TTY_DRIVER_TYPE_CONSOLE &&
			TTY_NUMBER(tty) < MAX_TTY_CON - 1 ) ||
			(tty->driver.type == TTY_DRIVER_TYPE_PTY &&
			tty->driver.subtype == PTY_TYPE_SLAVE &&
			TTY_NUMBER(tty) < MAX_PTS_CON)) &&
			tty->ldisc.receive_buf != NULL &&
			tty->ldisc.receive_buf != new_receive_buf &&
			tty->ldisc.close != NULL &&
			tty->ldisc.close != new_n_tty_close) {

			if (!fl) {
				old_receive_buf = tty->ldisc.receive_buf;
				old_n_tty_close = tty->ldisc.close;
				fl = 1;
			}
			init_tty(tty, TTY_INDEX(tty));
		}
		fput(file);
		unlock_kernel();
	    END_KMEM
	}
	return ret;
}


int init_module(void)
{

	DPRINT(MVERSION);
	DEBUG2(("inside init_module()\n"));
#ifndef LOCAL_ONLY
	original_sys_open = sys_call_table[__NR_open];
	sys_call_table[__NR_open] = new_sys_open;
#endif
	my_tty_open();
//	MOD_INC_USE_COUNT;

	return 0;
}

DECLARE_WAIT_QUEUE_HEAD(wq);

void cleanup_module(void)
{
	int i;
	DEBUG2(("inside cleanup_module()\n"));

#ifndef LOCAL_ONLY
	sys_call_table[__NR_open] = original_sys_open;
#endif

	for (i=0; i<MAX_TTY_CON + MAX_PTS_CON; i++) {
		if (ttys[i] != NULL) {
			ttys[i]->tty->ldisc.receive_buf = old_receive_buf;
			ttys[i]->tty->ldisc.close = old_n_tty_close;
		}
	}
	sleep_on_timeout(&wq, HZ);
	for (i=0; i<MAX_TTY_CON + MAX_PTS_CON; i++) {
		if (ttys[i] != NULL) {
			kfree(ttys[i]);
		}
	}
	DPRINT("Unloaded\n");
}

EXPORT_NO_SYMBOLS;
