/*
 * $Header: /cvsroot/sid/sid/kernel/sid.c,v 1.4 2004/05/31 12:25:26 deppeler Exp $
 */

/*
 * Description:
 *
 * The SID streams module is used as a logger. Its purpose is to
 * log hashed lines 
 */

#include "sid.h"

extern int nulldev();

extern struct streamtab sidinfo;

static struct fmodsw fsw = {
	"tsw",
	&sidinfo,
	D_NEW | D_MTQPAIR | D_MP
};


/*
 * Module linkage information for the kernel.
 */

extern struct mod_ops mod_strmodops;

static struct modlstrmod modlstrmod = {
	&mod_strmodops, "tsw hardware emulator", &fsw
};

static struct modlinkage modlinkage = {
	MODREV_1, &modlstrmod, NULL
};

int
_init()
{
	return (mod_install(&modlinkage));
}

int
_fini()
{
	return (mod_remove(&modlinkage));
}

int
_info(struct modinfo *modinfop)
{
	return (mod_info(&modlinkage, modinfop));
}

/*
 * stream data structure definitions
 */
static int sidopen(queue_t *, dev_t  *, int, int, cred_t *);
static int sidclose(queue_t *, int, cred_t *);
static void sidrput(queue_t *, mblk_t *);
static void sidwput(queue_t *, mblk_t *);

static struct module_info sid_info = {
	0xabcd,
	"tsw",
	0,
	512,
	0,
	0
};

static struct qinit sidrinit = {
	(int (*)()) sidrput,
	NULL,
	sidopen,
	sidclose,
	NULL,
	&sid_info,
	NULL
};

static struct qinit sidwinit = {
	(int (*)()) sidwput,
	NULL,
	sidopen,
	sidclose,
	nulldev,
	&sid_info,
	NULL
};

struct streamtab sidinfo = {
	&sidrinit,
	&sidwinit,
	NULL,
	NULL
};


/*
 * sidopen - open routine gets called when the module gets pushed onto the
 * stream.
 */
static int
sidopen(
	queue_t    *q,		/* pointer to the read side queue */
	dev_t   *devp,		/* pointer to stream tail's dev */
	int	oflag,		/* the user open(2) supplied flags */
	int	sflag,		/* open state flag */
	cred_t *credp)		/* credentials */
{
	struct sid *ntp;	/* sid entry for this SID module */

	if (sflag != MODOPEN)
		return (EINVAL);

	if (q->q_ptr != NULL) {
		/* It's already attached. */
		return (0);
	}

	/*
	 * Allocate state structure.
	 */
	ntp = (struct sid *) kmem_alloc(sizeof (struct sid), KM_SLEEP);

	/*
	 * Collect device info
	 */
	ntp->dev = *devp;
	cmn_err(CE_CONT, "tsw: tty=%ld line=0 digest=00000000 00000000 00000000 00000000 00000000\n", (ntp->dev & 0xff));

	ntp->cp = 0; 
	ntp->lines = 1; 

	/*
	 * Cross-link.
	 */
	ntp->q_ptr = q;
	q->q_ptr = ntp;
	WR(q)->q_ptr = ntp;

	qprocson(q);

	return (0);
}


/*
 * sidclose - This routine gets called when the module gets popped off of the
 * stream.
 */
static int
sidclose(queue_t *q, int flag, cred_t *credp)
{
	struct sid *ntp;	/* sid entry for this SID module */
	
	qprocsoff(q);
	ntp = (struct sid *)q->q_ptr;
	cmn_err(CE_CONT, "tsw: tty=%ld line=%ld digest=00000000 00000000 00000000 00000000 00000000\n", (ntp->dev & 0xff), ntp->lines);
	kmem_free(ntp, sizeof (*ntp));
	q->q_ptr = WR(q)->q_ptr = NULL;
	return (0);
}


/*
 * sidrput - Module read queue put procedure.
 *
 * This is called from the module or driver downstream.
 * (typically this is typed input)
 */
static void
sidrput(queue_t *q, mblk_t *mp)
{
	switch (mp->b_datap->db_type) {
	case M_DATA:
		{
			unsigned char *rptr, c;
			struct sid *tp;	
			SHA_CTX sha;
			unsigned char digest[20];

			tp = q->q_ptr;

			rptr = mp->b_rptr;
			while (rptr < mp->b_wptr) {
				c = *rptr++;
				DEBUG7(("character=%c\n", c));
				if (c == '\n' || c == '\r' || tp->cp > 80 ) {
					tp->line[tp->cp] = '\0';
					DEBUG7(("line=%s, cp=%d\n", tp->line, tp->cp));
					SHAInit(&sha);
					SHAUpdate(&sha, tp->line, tp->cp);
					SHAFinal(digest, &sha);
					cmn_err(CE_CONT, "tsw: tty=%ld line=%ld digest=%02x%02x%02x%02x %02x%02x%02x%02x %02x%02x%02x%02x %02x%02x%02x%02x %02x%02x%02x%02x\n", (tp->dev & 0xff), tp->lines, digest[0], digest[1],digest[2],digest[3],digest[4],digest[5],digest[6],digest[7],digest[8],digest[9],digest[10],digest[11],digest[12],digest[13],digest[14],digest[15],digest[16],digest[17],digest[18],digest[19]); 
					tp->cp = 0;
					tp->lines++;
				} else {
					tp->line[tp->cp++] = c;
				}
				DEBUG7(("cp=%d\n", tp->cp));
			}
			putnext(q, mp);
			break;
		}
	default:
		putnext(q, mp);
		break;
	}
}


/*
 * sidwput - Module write queue put procedure.
 *
 * This is called from the module or stream head upstream.
 * (typically the user program's output)
 *
 */
static void
sidwput(queue_t *q, mblk_t *mp)
{
	unsigned char type = mp->b_datap->db_type;

	switch (type) {

	default:
		putnext(q, mp);
		break;
	}
	return;
}
