#!/bin/sh -e
#
# $Header: /cvsroot/sid/sid/package.sh,v 1.15 2005/01/08 19:37:48 deppeler Exp $
#
# This script builds Solaris packages of sid.
# does it work? yes.
# is is nice? noooo.
#

VERSION=`awk '/^source: / { print $2 }' version`

SED="/bin/sed"
SVERSION=`echo $VERSION | $SED -e 's/\.//g'`

INSTALL_PATH_32="/usr/kernel/strmod"
KERNEL_MOD="kernel"
KERNEL_MOD_32="tsw"
INSTALL_PATH_64="/usr/kernel/strmod/sparcv9"
KERNEL_MOD_64="sparcv9/tsw"
INSTALL_PATH_user="/usr/local/bin/"
USER_PATH="user"
USER_PROG="tsw"
INSTALL_PATH_config="/etc/"
CONFIG_PATH="user"
CONFIG_FILE_IN="test.in"
CONFIG_FILE_DEST="tsw.conf.example"
CONFIG_FILE="tsw.conf"
INSTALL_PATH_autopush="/etc/"
AUTOPUSH_PATH="kernel"
AUTOPUSH_FILE="iu.ap"
AUTOPUSH_DEST_FILE="iu.ap.tsw"

KERNEL_BASE_PKGNAME="tswk"
USER_BASE_PKGNAME="tswu"

INSTALL_PATH_initdtsw="/etc/init.d"
INSTALL_FILE_initdtswk=${KERNEL_BASE_PKGNAME}
INSTALL_FILE_initdtswu=${USER_BASE_PKGNAME}

pwd=`pwd`

#
# build kernel part
#
if [ -x /usr/bin/isainfo ]
then
        isa=`isainfo -b`
else
        isa=32
fi

cd kernel
make -f Makefile.solaris clean
if [ $isa -eq 64 ]
then
        make -f Makefile.solaris all64
else
        make -f Makefile.solaris all
fi

cd $pwd

#
# build user part
#
cd user
make -f Makefile.solaris clean
make -f Makefile.solaris

cd $pwd


cat > pkginfo << END_PKGINFO
#
#
PKG=${KERNEL_BASE_PKGNAME}${SVERSION}
NAME=SID Kernel ${VERSION}
VERSION=${VERSION}
ARCH=sparc
CLASSES=none
CATEGORY=application
BASEDIR=/
DESC=SID - Shell Intrusion Detection Kernel Part
VENDOR=none
EMAIL=harald@deppeler.org
MAXINST=1
OWN=root
GRP=other
END_PKGINFO

cat > copyright << END_COPYRIGHT

        SID Distribution v${VERSION}

        Copyright 2004, Harald Deppeler


END_COPYRIGHT

PKG_NAME=`grep 'PKG' pkginfo | cut -c 5-`
rm -rf $PKG_NAME
mkdir -p $PKG_NAME
mkdir $PKG_NAME/root
mkdir $PKG_NAME/install
touch $PKG_NAME/install/depend

cp copyright pkginfo $PKG_NAME

#
# create minimal directory structure
#
> tmp_pkgmap.$PKG_NAME
#
# create pkgmap entries for INSTALL_PATH
#
for tmp_INSTALL_PATH in ${INSTALL_PATH_64}
do
        all_chars=`echo $tmp_INSTALL_PATH | wc -c`
        part_chars=`echo $tmp_INSTALL_PATH | tr -d '/' | wc -c`
        while [ `expr $all_chars - $part_chars` -gt 1 ] ;
        do
                echo "1 d none $tmp_INSTALL_PATH 0755 root root" >> tmp_pkgmap.$PKG_NAME
                mkdir -p $PKG_NAME/root/$tmp_INSTALL_PATH
                tmp_INSTALL_PATH=`echo $tmp_INSTALL_PATH | sed -e 's!/[^/]*$!!'`
                all_chars=`echo $tmp_INSTALL_PATH | wc -c`
                part_chars=`echo $tmp_INSTALL_PATH | tr -d '/' | wc -c`
        done
done
sort tmp_pkgmap.$PKG_NAME > pkgmap.$PKG_NAME
rm -f tmp_pkgmap.$PKG_NAME

#
# create kernel modules
#
size=`ls -l ${KERNEL_MOD}/${KERNEL_MOD_32} | awk '{ print $5 }'`
sum=`sum ${KERNEL_MOD}/${KERNEL_MOD_32} | awk '{ print $1 }'`
echo "1 f none ${INSTALL_PATH_32}/${KERNEL_MOD_32} 0700 root root $size $sum 1035989733" >> pkgmap.$PKG_NAME
cp ${KERNEL_MOD}/${KERNEL_MOD_32} $PKG_NAME/root/${INSTALL_PATH_32}
if [ $isa -eq 64 ]
then
	size=`ls -l ${KERNEL_MOD}/${KERNEL_MOD_64} | awk '{ print $5 }'`
	sum=`sum ${KERNEL_MOD}/${KERNEL_MOD_64} | awk '{ print $1 }'`
	echo "1 f none ${INSTALL_PATH_64}/${KERNEL_MOD_32} 0700 root root $size $sum 1035989733" >> pkgmap.$PKG_NAME
	cp ${KERNEL_MOD}/${KERNEL_MOD_64} $PKG_NAME/root/${INSTALL_PATH_64}
fi


#
# create autopush
#

mkdir -p ${PKG_NAME}/root/${INSTALL_PATH_autopush}


size=`ls -l ${AUTOPUSH_PATH}/${AUTOPUSH_FILE} | awk '{ print $5 }'`
sum=`sum ${AUTOPUSH_PATH}/${AUTOPUSH_FILE} | awk '{ print $1 }'`
echo "1 f none ${INSTALL_PATH_autopush}/${AUTOPUSH_DEST_FILE} 0700 root root $size $sum 1035989733" >> pkgmap.$PKG_NAME
cp ${AUTOPUSH_PATH}/${AUTOPUSH_FILE} $PKG_NAME/root/${INSTALL_PATH_autopush}/${AUTOPUSH_DEST_FILE}

#
# kernel module startup script
#
mkdir -p ${PKG_NAME}/root/${INSTALL_PATH_initdtsw}
cat > ${PKG_NAME}/root/${INSTALL_PATH_initdtsw}/${INSTALL_FILE_initdtswk} << EOF_INITD
#!/bin/sh

AUTOPUSHFILE=${INSTALL_PATH_autopush}${AUTOPUSH_DEST_FILE}
AUTOPUSH="/usr/sbin/autopush"
MODLOAD="/usr/sbin/modload"
MODUNLOAD="/usr/sbin/modunload"
MODINFO="/usr/sbin/modinfo"
GREP="/usr/bin/grep"
AUTOPUSH_MAJOR=24
AUTOPUSH_MINOR=0
TAIL="/usr/bin/tail"
AWK="/usr/bin/awk"

testmod=\`\$MODINFO | \$GREP ${KERNEL_MOD_32} | \$AWK '{ print \$6 }'\`
testautopush=\`\$AUTOPUSH -g -M \${AUTOPUSH_MAJOR} -m \${AUTOPUSH_MINOR} 2>&1 | \$TAIL -1 | \$AWK '{ print \$4 }'\`

case "\$1" in
        start)
                echo "starting ${KERNEL_MOD_32} - \c"

		if [ "x\$testmod" = "x${KERNEL_MOD_32}" ] 
		then
			echo "(kernel module already loaded) \\c"
		else
                	\$MODLOAD -p strmod/${USER_PROG} && echo "modload done \\c" || echo "modload failed \\c"
		fi

		if [ "x\$testautopush" = "x${KERNEL_MOD_32}" ]
		then
			echo "(autopush already configured) \\c"
		else
                	\$AUTOPUSH -f \$AUTOPUSHFILE && echo "autopush done" || echo "autopush failed"
		fi
                ;;
        stop)
                echo "stopping ${KERNEL_MOD_32} - \c"
		
		if [ "x\$testmod" = "x${KERNEL_MOD_32}" ]
		then
                	modnumber=\`\$MODINFO | grep ${KERNEL_MOD_32} | \$AWK '{ print \$1 }'\`
                	[ -z "\$modnumber" ] || \$MODUNLOAD -i \$modnumber && echo "modunload done \\c" || echo "modunload failed \\c"
		else
			echo "(kernel module was not loaded) \\c"
		fi

		if [ "x\$testautopush" = "x${KERNEL_MOD_32}" ]
		then
               		\$AUTOPUSH -r -M \${AUTOPUSH_MAJOR} -m \${AUTOPUSH_MINOR} && echo "autopush done" || echo "autopush failed"
		else
			echo "(autopush was not configured) \\c"
		fi
                ;;
        restart)
                \$0 stop
                sleep 1
                \$0 start
                ;;
        *)
                echo "Usage: \$0 { start | stop | restart }"
                exit 1
                ;;
esac
exit 0
EOF_INITD

size=`ls -l ${PKG_NAME}/root/${INSTALL_PATH_initdtsw}/${INSTALL_FILE_initdtswk} | awk '{ print $5 }'`
sum=`sum ${PKG_NAME}/root/${INSTALL_PATH_initdtsw}/${INSTALL_FILE_initdtswk} | awk '{ print $1 }'`
echo "1 f none ${INSTALL_PATH_initdtsw}/${INSTALL_FILE_initdtswk} 0700 root root $size $sum 1035989733" >> pkgmap.$PKG_NAME

#
# create pkgmap
#
size=`du -s ${PKG_NAME}/root | awk '{ print $1 }'`
# create pkgmap
echo "size is $size"
echo ": 1 $size" > $PKG_NAME/pkgmap
# fix pkginfo entry
size=`ls -l pkginfo | awk '{ print $5 }'`
sum=`sum pkginfo | awk '{ print $1 }'`
echo "1 i pkginfo $size $sum 1035989733" >> pkgmap.$PKG_NAME
echo "1 i depend 0 0 1035989733" >> pkgmap.$PKG_NAME
cat pkgmap.$PKG_NAME >> $PKG_NAME/pkgmap
rm pkgmap.$PKG_NAME

pkgtrans . ${KERNEL_BASE_PKGNAME}_${VERSION}_`uname -r`_`uname -p`.pkg $PKG_NAME

rm -f copyright pkginfo
rm -rf $PKG_NAME

cat > pkginfo << END_PKGINFO
#
#
PKG=${USER_BASE_PKGNAME}${SVERSION}
NAME=SID User ${VERSION}
VERSION=${VERSION}
ARCH=sparc
CLASSES=none
CATEGORY=application
BASEDIR=/
DESC=SID - Shell Intrusion Detection User Part
VENDOR=none
EMAIL=harald@deppeler.org
MAXINST=1
OWN=root
GRP=other
END_PKGINFO

cat > copyright << END_COPYRIGHT

        SID Distribution v${VERSION}

        Copyright 2004, Harald Deppeler


END_COPYRIGHT

PKG_NAME=`grep 'PKG' pkginfo | cut -c 5-`
rm -rf $PKG_NAME
mkdir -p $PKG_NAME
mkdir $PKG_NAME/root
mkdir $PKG_NAME/install
touch $PKG_NAME/install/depend

cp copyright pkginfo $PKG_NAME

#
# create minimal directory structure
#
> tmp_pkgmap.$PKG_NAME
#
# create pkgmap entries for INSTALL_PATH
#
for tmp_INSTALL_PATH in ${INSTALL_PATH_user}
do
        all_chars=`echo $tmp_INSTALL_PATH | wc -c`
        part_chars=`echo $tmp_INSTALL_PATH | tr -d '/' | wc -c`
        while [ `expr $all_chars - $part_chars` -gt 1 ] ;
        do
                echo "1 d none $tmp_INSTALL_PATH 0755 root root" >> tmp_pkgmap.$PKG_NAME
                mkdir -p $PKG_NAME/root/$tmp_INSTALL_PATH
                tmp_INSTALL_PATH=`echo $tmp_INSTALL_PATH | sed -e 's!/[^/]*$!!'`
                all_chars=`echo $tmp_INSTALL_PATH | wc -c`
                part_chars=`echo $tmp_INSTALL_PATH | tr -d '/' | wc -c`
        done
done
sort tmp_pkgmap.$PKG_NAME > pkgmap.$PKG_NAME
rm -f tmp_pkgmap.$PKG_NAME


#
# create user part
#

mkdir -p ${PKG_NAME}/root/${INSTALL_PATH_config}
size=`ls -l ${CONFIG_PATH}/${CONFIG_FILE_IN} | awk '{ print $5 }'`
sum=`sum ${CONFIG_PATH}/${CONFIG_FILE_IN} | awk '{ print $1 }'`
echo "1 f none ${INSTALL_PATH_config}/${CONFIG_FILE_DEST} 0700 root root $size $sum 1035989733" >> pkgmap.$PKG_NAME
cp ${CONFIG_PATH}/${CONFIG_FILE_IN} $PKG_NAME/root/${INSTALL_PATH_config}/${CONFIG_FILE_DEST}

size=`ls -l ${USER_PATH}/${USER_PROG} | awk '{ print $5 }'`
sum=`sum ${USER_PATH}/${USER_PROG} | awk '{ print $1 }'`
echo "1 f none ${INSTALL_PATH_user}/${USER_PROG} 0700 root root $size $sum 1035989733" >> pkgmap.$PKG_NAME
cp ${USER_PATH}/${USER_PROG} $PKG_NAME/root/${INSTALL_PATH_user}

#
# startup script
#
mkdir -p ${PKG_NAME}/root/${INSTALL_PATH_initdtsw}
cat > ${PKG_NAME}/root/${INSTALL_PATH_initdtsw}/${INSTALL_FILE_initdtswu} << EOF_INITD
#!/bin/sh

DAEMON=${INSTALL_PATH_user}/${USER_PROG}
CONFIGFILE=${INSTALL_PATH_config}/${CONFIG_FILE}

case "\$1" in
        start)
                echo "starting ${USER_PROG} - \c"
		[ -r \${CONFIGFILE} ] || \
			{ echo "configuration file \${CONFIGFILE} missing or unreadable"; \
			  exit 2; }
                \$DAEMON -c \$CONFIGFILE > /dev/null 2>&1 && echo "daemon done" || echo "daemon failed"
                ;;
        stop)
                echo "stopping ${USER_PROG} - \c"
                pid=\`ps -ef | grep tsw | egrep -v "(grep|init.d)" | awk '{ print \$2 }'\`
                [ -z "\$pid" ] || kill \$pid && echo "daemon done" || echo "daemon failed"
                ;;
        restart)
                \$0 stop
                sleep 1
                \$0 start
                ;;
        *)
                echo "Usage: \$0 { start | stop | restart }"
                exit 1
                ;;
esac
exit 0
EOF_INITD

size=`ls -l ${PKG_NAME}/root/${INSTALL_PATH_initdtsw}/${INSTALL_FILE_initdtswu} | awk '{ print $5 }'`
sum=`sum ${PKG_NAME}/root/${INSTALL_PATH_initdtsw}/${INSTALL_FILE_initdtswu} | awk '{ print $1 }'`
echo "1 f none ${INSTALL_PATH_initdtsw}/${INSTALL_FILE_initdtswu} 0700 root root $size $sum 1035989733" >> pkgmap.$PKG_NAME

#
# create pkgmap
#
size=`du -s ${PKG_NAME}/root | awk '{ print $1 }'`
# create pkgmap
echo "size is $size"
echo ": 1 $size" > $PKG_NAME/pkgmap
# fix pkginfo entry
size=`ls -l pkginfo | awk '{ print $5 }'`
sum=`sum pkginfo | awk '{ print $1 }'`
echo "1 i pkginfo $size $sum 1035989733" >> pkgmap.$PKG_NAME
echo "1 i depend 0 0 1035989733" >> pkgmap.$PKG_NAME
cat pkgmap.$PKG_NAME >> $PKG_NAME/pkgmap
rm pkgmap.$PKG_NAME

pkgtrans . ${USER_BASE_PKGNAME}_${VERSION}_`uname -r`_`uname -p`.pkg $PKG_NAME

rm -f copyright pkginfo
rm -rf $PKG_NAME
exit 0
