/* $Header: /cvsroot/sid/sid/user/c_tsw.h,v 1.32 2004/11/05 12:43:33 deppeler Exp $ */

#ifndef _C_TSW_H
#define _C_TSW_H

#include <stdio.h>
#include <assert.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <unistd.h>
#include <strings.h>
#include <string.h>
#include <libgen.h>
#include <stdlib.h>
#include <time.h>
#include <signal.h>

#ifndef TRUE
  #define FALSE	0
  #define TRUE	( !FALSE )
#endif /* TRUE */

/* Size of atomic reads.  */
#ifndef BUFSIZ
# define BUFSIZ (512 * 8)
#endif

#ifndef MIN
# define MIN(a,b) (((a) < (b)) ? (a) : (b))
#endif

#ifndef CHAR_BIT
# define CHAR_BIT 8
#endif

#define TYPE_MINIMUM(t) ((t) (TYPE_SIGNED (t) \
                              ? ~ (t) 0 << (sizeof (t) * CHAR_BIT - 1) : (t) 0))
#define TYPE_MAXIMUM(t) ((t) (~ (t) 0 - TYPE_MINIMUM (t)))

#ifndef OFF_T_MAX
# define OFF_T_MAX TYPE_MAXIMUM (off_t)
#endif

#define COPY_TO_EOF OFF_T_MAX


#define MAX_LINE_LENGTH 256
#define SHA1_HASH_SIZE 20
#define MAX_COMMANDS 256
#define NR_OF_WEEKDAYS 7
#define HASH_LENGTH 45	    		/* hash + spaces + '\0' */
#define HASH_ANNOUNCEMENT_LENGTH 7	/* digest= */
#define TTY_ANNOUNCEMENT_LENGTH 4	/* tty= */
#define TTY_MIN_LENGTH 1
#define MIN_DIGEST_LINE_LENGTH 70	/* see comment below */
#define FMT_MATCH_HOST "%*s %*d %*d:%*d:%*d %s "
#define FMT_MATCH_TSW " tsw: tty=%d line=%d digest=%8[0123456789abcdef] %8[0123456789abcdef] %8[0123456789abcdef] %8[0123456789abcdef] %8[0123456789abcdef]"
#define TSW_OPEN_CLOSE "00000000 00000000 00000000 00000000 00000000"
#define WEEK_SECONDS 604800 		/* (7*24*60*60) */
#define DAY_SECONDS  86400		/* (24*60*60) */
#define HOUR_SECONDS 3600		/* (60*60) */
#define MINUTE_SECONDS 60

/*
                                            70        60        50    44  40        30        20        10
    09876543210987654321098765432109876543210987654321098765432109876543210987654321098765432109876543210987654321
Apr  2 21:07:15 blade tsw: [ID 769231 kern.notice] tsw: tty=11 digest=e2cef1f8 dcab8cbc 6d6c7c74 06a22505 25b7be9f
                                                               1234567
Feb 23 15:48:12 debianvanilla kernel: tsw: tty=0 line=387 digest=8a90d28c 8825412f 6008da90 f8770c2b 05282358
 */

enum {
	NONE = 10000,
	INTERACTIVE = 10001,
	PASSWORD = 10002,
	SET = 10003,
	UNSET = 10004,
	PERIODIC = 10005,
	SINGULAR = 10006,
	WITHIN_HOST_DEF = 10007,    /* warning: also defined in lexer */
	WITHIN_COMMAND_DEF = 10008,  /* warning: also defined in lexer */
	WITHIN_GLOBAL_DEF = 10009,  /* warning: also defined in lexer */
       	GLOBAL = 10010,
	NORMAL_CMD = 10011,
	ANY_CMD = 10012,
	SHA1 = 10013
};


/*******************/
/* data structures */
/*******************/

/* tsw_time holds a list of time specifications */
struct tsw_time { 
	int 			type;	/* periodic (weekday) or singular */
	int 			weekday[NR_OF_WEEKDAYS];
	time_t 			ctime;  /* point in time start time (unused) */
	int 			start_hour;
	int 			start_minute;
	int 			cont_day;
	int 			cont_hour;
	int 			cont_minute;
	int 			cont_second;
	struct se_time	*se;    /* once calculated start and end times
(from time specs above) to faciliate time checking at run time */
	struct tsw_time		*next;
};

/* hour minute second */
struct hms {
	int hour;
	int minute;
	int second;
};

/* se_time holds a list of start and end time specs 
   (these are calculated from the specs in tsw_time)
 */
struct se_time {
	int 			weekday[NR_OF_WEEKDAYS];
	struct hms			start;
	struct hms			end;
	struct se_time	*next;
};

struct file_spec
{
        int 	fd;
	off_t 	size;
};

/* host_cft_t holds a list of host definitions */
struct host_cfg {
	char	name[MAX_LINE_LENGTH]; /* hostname (as it appears in logfile */
	short  	admin_mode;	       /* administration mode (overrides global) */ 
	unsigned char	admin_digest[SHA1_HASH_SIZE]; /* password hash (calculated) */
	int	max_failures; /* TODO: number of trials for admin mode */
	struct 	tsw_time *automated_time; /* whence are automated actions */
	char	automated_action[MAX_LINE_LENGTH]; /* full command line */
	char	syslog_message[MAX_LINE_LENGTH];   /* intrusion log alert */
	char	syslog_facility[MAX_LINE_LENGTH];  /* facility for the above */
	int	facility;
	char	syslog_priority[MAX_LINE_LENGTH];  /* priority for the above */
	int	priority;
	char	syslog_tag[MAX_LINE_LENGTH];       /* 'tag' for the above */
	struct	host_cfg *next;
};

/* config holds a configuration */
struct config {
	struct host_cfg	*host;    /* list of host definitions */
	short 		admin_mode; /* global administration mode type */
	unsigned char	admin_digest[SHA1_HASH_SIZE]; /* password hash (calculated) */
	int		max_failures; /* number of trials for admin mode */
	char		logfile[MAX_LINE_LENGTH]; /* tail this logfile */
};

/* state holds a list of open terminals */
struct state {
	char		*host;	/* hostname */
	int		pty;	/* pseudo terminal number */
	short		in_sequence; /* expect the next cmd in sequence? */
	unsigned char	*next_expected_hash;
	int		failures;
	int		automated_action; /* are we within an automated action */
	long		hashes;		  /* number of hashes so far */
	short		maintenance; /* state in maintenance? */
	struct state	*next;
};

/* cmd_line_t holds parsed cmd line args */
struct cmd_line_t {
	char		*file; /* configuration file */
	int		debug; /* debug level */
	int		s;     /* flag for creating a sha1 hash */
	int		q;     /* flag for quiet mode */
	int		t;     /* flag for syntax check */
};


int lex_state;

/********************* */
/* function prototypes */
/********************* */

/* used for tailing on the logfile */
int open_log(struct file_spec *s_file);
int proc_buf(char *buf, int bytes_read);
int tail(struct file_spec s_file); 
long dump_remainder (int fd, off_t n_bytes);
int tail_on_file(struct config *c);

/* used to parse logfile lines */
int proc_line(char *line, int length);
int check_digest_validity(const char *hash, const char *host, int pty);
int pty_hash(const char *line, int length);
int find_host_in_line(const char *line, int length, char *host, int dest_length);
int find_tsw_in_line(const char *line, char *hash, int hash_length, int *tty, long *linenumber);
int fix_open_close(const char *hash, const char *host, int tty, int ln);
int check_host(const char *line, int line_length, char *host, int host_length);
int chk_hash(const char *hash, const char *host, int tty);

/* various helper functions */
/* (aren't they *all* helper functions?!) */
void dprintf(int l, char *fmt, ...);
void setprogname(char *str);
char *estrdup(const char *s);
char *getprogname(void);
void eprintf(int e, char *fmt, ...);
void dprintf(int l, char *fmt, ...);
void collect_children(void);

/* (de-)allocate data structures */
void free_cmd_line_t(struct cmd_line_t *c);
void free_host_cfg(struct host_cfg *c);
void free_config(struct config *c);
void free_and_exit(int);

struct tsw_time *new_time(void);
struct tsw_time * add_time_front(struct tsw_time *listp, struct tsw_time *newp);
struct host_cfg *addfront(struct host_cfg *listp, struct host_cfg *newp);
void *ecalloc(size_t nelem, size_t elsize);
struct host_cfg *new_host_cfg(void);
struct host_cfg *find_host(const struct host_cfg *h, const char *name);
int add_cmd(struct config *c, char *s, int mode);

int hex2bin_hash(char *s, unsigned char *digest);
int hex2bin(char h1, char h2);
int cleanup_hex(char *s1, char *s2);

struct state *add_state(struct state *s, const char *host, int pty);
struct state *rem_state(struct state *state, const char *host, int pty);
struct state *add_state_l(struct state *listp, struct state *newp);
struct state *new_state(void);
struct state *get_state(const struct state *s, const char *host, int pty);

struct se_time	*new_se_time(void);
struct se_time	*add_se_time_front(struct se_time *listp, struct se_time *newp);

int add_start(struct tsw_time *tsw_time, int day, struct se_time *se);
int add_end(struct tsw_time *time_spec, int day, int hour, int minute, int second);

/* various find and value check functions for our data structures */
int chk_maint(struct config *config, struct state *s, const char *hash);

int time_ok(struct tsw_time *tsw_time);

/* functions to preprocess configuration file data */
int preprocess_config(struct config *config);
int preprocess_time_spec(struct config *config);
int preprocess_syslog_spec(struct config *config);
int preprocess_time(struct tsw_time *time_spec, int day);
long calc_delta(struct tsw_time *time_spec); 

/* maintenance mode related */
int maint_pass(struct config *config);
char *read_pass(const char *s);
int gen_pass(void);

/* 'intruder found' functions */
int automated_action(struct state *state, struct host_cfg *host_cfg);
int log_alert(struct host_cfg *host_cfg);
int log_automated_action(struct host_cfg *host_cfg);
int log_ok(struct host_cfg *host_cfg);

/* cmd line args and configuration */
int cmdargs_check(struct cmd_line_t *cmd_line_opts);
int get_config(char *filename, int i);
int read_config(char *filename);
int activate_config(void);
struct cmd_line_t *cmd_parse(int argc, char **argv);
void usage(void);
int become_daemon(const struct cmd_line_t *cmd_line_opts);

/* globals - are these evil by nature? */
struct config	*config; 
struct config	*c_test; 	/* used for the parser */
char   		*progname;   	/* program name */
struct state		     *state;
struct cmd_line_t *cmd_line_opts;

/* signal related */
void sid_sighup(int sign);

#endif	/* _C_TSW_H */
