/* $Header: /cvsroot/sid/sid/user/sid_memory.c,v 1.2 2004/11/04 09:27:34 deppeler Exp $ */

#include "c_tsw.h"

/* free_host_cfg() - deallocate host configuration list 
 * @c: pointer to host_cfg linked list
 */ 
void free_host_cfg(struct host_cfg *c)
{
	struct host_cfg *next;
	struct tsw_time *next_a_t; 	/* next automated time */

	for (; c != NULL; c = next) {

		/* free automated_time list */
		for (; c->automated_time != NULL;
		     c->automated_time = next_a_t) {
			next_a_t = c->automated_time->next;
			free(c->automated_time);
		}

		next = c->next;
		free(c);
	}
}

/* free_config() - deallocate configuration
 * @c: pointer to config struct to free
 */
void free_config(struct config *c)
{
	if (c != NULL) {
		free_host_cfg(c->host);
		free(c);
	}
}

/* new_time() - return newly allocated time specification structure */
struct tsw_time *new_time(void)
{
	struct tsw_time *newp;

	newp = (struct tsw_time *)ecalloc(1, sizeof(struct tsw_time));
	newp->next = NULL;
	return newp;
}

/* add_time_front(): add time_spec newp in front of listp, return new list head 
 * @listp: pointer to current list head
 * @newp: new time_spec to add at front
 */
struct tsw_time *add_time_front(struct tsw_time *listp, struct tsw_time *newp)
{
	newp->next = listp;
	return newp;
}

/* addfront() -  add host_cfg newp to front of listp, return new list head 
 * @listp: pointer to current list head
 * @newp: new host_cfg to add at front
 */
struct host_cfg *addfront(struct host_cfg *listp, struct host_cfg *newp)
{
	newp->next = listp;
	return newp;
}

/* ecalloc() - malloc and report if error 
 * @nelem: number of elements
 * @elsize: size of element
 */
void *ecalloc(size_t nelem, size_t elsize)
{
	void *p;

	p = calloc(nelem, elsize);
	if (NULL == p)
		eprintf(2, "malloc of %u bytes failed:", nelem * elsize);
	return p;
}

/* new_host_cfg() - allocate new host configuration */
struct host_cfg *new_host_cfg(void)
{
	struct host_cfg *newp;

	newp = (struct host_cfg *)ecalloc(1, sizeof(struct host_cfg));
	newp->next = NULL;
	return newp;
}

/* new_state() - return newly allocated state */
struct state *new_state(void)
{
	struct state *newp;

	newp = (struct state *)ecalloc(1, sizeof(struct state));
	newp->next = NULL;
	return newp;
}

/* add_state_l() - add state in front (left) of list 
 * @listp: current list pointer
 * @newp: new state to add 
 */
struct state *add_state_l(struct state *listp, struct state *newp)
{
	newp->next = listp;
	return newp;
}

/* free_cmd_line_t() - free the specified command line structure
 * @c: pointer to command line structure
 */
void free_cmd_line_t(struct cmd_line_t *c)
{
	if (NULL == c)
		return;
	free(c->file);
	free(c);
}

/* free_and_exit() - free memory and exit with return code i 
 * @i: return code to exit with
 */
void free_and_exit(int i)
{
	free_config(c_test);
	free_config(config);
	free(progname);
	free_cmd_line_t(cmd_line_opts);
	exit(i);
}

/* estrdup() - return pointer to duplicate a string (report if error)
 * @s	string to duplicate
 */
char *estrdup(const char *s)
{
	char *t;

	t = (char *)malloc(strlen(s) + 1);
	if (NULL == t)
		eprintf(2, "estrdup(\"%.20s\") failed:", s);
	strcpy(t, s);
	return t;
}

/* new_se_time() - return pointer to new 'start- end' time spec */
struct se_time *new_se_time(void)
{
	struct se_time *newp;

	newp = (struct se_time *)ecalloc(1, sizeof(struct se_time));
	newp->next = NULL;
	return newp;
}

/* add_se_time_front() - add new 'start-end' time structure, return new list head 
 * @listp: pointer to current list
 * @newp: new 'start-end' time struct to add to the front of the list 
 */
struct se_time *add_se_time_front(struct se_time *listp, struct se_time *newp)
{
	newp->next = listp;
	return newp;
}

/* indent -kr -i8 -ts8 -sob -l80 -ss -ncs */
