%{

/* $Header: /cvsroot/sid/sid/user/tswlex.l,v 1.8 2004/07/19 19:38:34 deppeler Exp $ */

#include "tswyac.h"
#include <string.h>

extern int lineno;
extern int lex_state;

enum {
	WITHIN_HOST_DEF = 10007,
	WITHIN_COMMAND_DEF = 10008,
	WITHIN_GLOBAL_DEF = 10009
};
%}

ws        [ \t]+
comment   #.*
qstring   \"[^\"\n]*[\"\n]
id        [a-zA-Z][a-zA-Z0-9]*
string    [a-zA-Z]+
nl        \n
number	  [0-9]+


%%

{ws}      ;
{comment} ;
{qstring} { yylval.string = strdup(yytext+1); /* skip open quote */
	    if(yylval.string[yyleng-2] != '"')
		printf("warning: unterminated character string ...");
		/* FIXME: well, do something on this error */
	    else
		yylval.string[yyleng-2] = '\0'; /* remove close quote */
	    return QSTRING;
	  }

host: {	lex_state = WITHIN_HOST_DEF; return HOST; }

admin: { return ADMIN; }

admin_attempts: { return ADMIN_ATTEMPTS; }

logfile: { lex_state = WITHIN_GLOBAL_DEF; return LOGFILE; }

global 	 |
password |
none	 |
sha1	 |
interactive { yylval.string = strdup(yytext);
	      return ADMINMODE; 
	    }


automated_time: { return AUTOMATED_TIME; }

automated_action: { return AUTOMATED_ACTION; }

syslog: { return SYSLOG; }

message: { return SYSLOG_MESSAGE; } 

priority: { return SYSLOG_PRIORITY; }

facility: { return SYSLOG_FACILITY; }

tag: { return SYSLOG_TAG; }

y |
yes { return YES; }

n |
no { return NO; }

Mo |
mo |
Monday |
monday   { yylval.weekday = 1;
            return WEEKDAY;
          }	    

Tu |
tu |
Tuesday |
tuesday   { yylval.weekday = 2;
            return WEEKDAY;
          }	    
We |
we |
Wednesday |
wednesday { yylval.weekday = 3;
            return WEEKDAY;
          }	    
Th |
th |
Thursday |
thursday  { yylval.weekday = 4;
            return WEEKDAY;
          }	    
Fr |
fr |
Friday |
friday   { yylval.weekday = 5;
            return WEEKDAY;
          }	    
Sa |
sa |
Saturday |
saturday  { yylval.weekday = 6;
            return WEEKDAY;
          }	    
Su |
su |
Sunday |
sunday    { yylval.weekday = 0;
            return WEEKDAY;
          }	    
h |
H |
Hour |
Hours |
hour |
hours      { 
            return HOUR;
           }	    

s |
S |
sec |
Second |
second |
Seconds |
seconds    { 
            return SECOND;
           }	    

m |
M |
min |
Minute |
minute |
Minutes |
minutes    { 
            return MINUTE;
           }	    

d |
D |
Day |
day |
days |
Days    { 
            return DAY;
           }	    

{number} { yylval.time = atoi(strdup(yytext));
	   /* printf("number: text %s, atoi %d\n", yytext, yylval.time); */
           return DIGITS;
	 }		    	

{string} { yylval.string = strdup(yytext);
             return STRING;
	   }


{nl}      { lineno++; }
.         { return yytext[0]; }

%%
void yyerror(char *s)
{
	printf("error: %s\n", s);
}
