%{
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include "c_tsw.h"
#include "sha1_kernel.h"

/* $Header: /cvsroot/sid/sid/user/tswyac.y,v 1.14 2004/09/13 17:29:47 deppeler Exp $ */
    
char *act_str;   /* extra argument for an action */
char *cmd_str;   /* extra argument for command */
char *item_str;  /* extra argument for 
                  * item description */

extern struct host_cfg *cur_host;	/* points to current host configuration */
extern struct tsw_time *cur_time;		/* used while scanning */

/* function prototypes */
short set_admin_mode(char * mode);

%}

%union {
    char    *string;     /* string buffer */
    int	    weekday;	 /* weekday */
    int     time;
    int    cmd;          /* command value */ } 

%token <string> QSTRING 
%token <string> ADMINMODE 
%token <time> DIGITS
%token <time> ADMIN_ATTEMPTS

%token HOST ADMIN AUTOMATED_TIME 
%token YEARLY_DATE_TIME_SPEC 
%token WEEKDAY HOUR SECOND MINUTE DAY STRING AUTOMATED_ACTION SYSLOG
%token SYSLOG_MESSAGE SYSLOG_PRIORITY SYSLOG_TAG 
%token YES NO 
%token SYSLOG_FACILITY LOGFILE 


%%

configuration: globalconfig hostconfigs
	;

globalconfig: logfile adminmode 
	;

hostconfigs: hostconfig
	| hostconfigs hostconfig
	;

hostconfig: hostname directives
        ;

hostname:  HOST QSTRING 
{
	dprintf(2, "parser: found host \"%s\"", $2); 
	dprintf(4, "cur_host is at %x, c_test->host is at %x.", cur_host, c_test->host);

	cur_host = addfront(cur_host, new_host_cfg());
	c_test->host = cur_host;

	dprintf(4, "cur_host is at %x, c_test->host is at %x.", cur_host, c_test->host);
	dprintf(5, "cur_host->next is at %x.", cur_host->next);
	strncat(cur_host->name, $2, MAX_LINE_LENGTH);
	dprintf(3, "parser: cur_host->name \"%s\"", cur_host->name); 
}
;

directives: /* empty */
	| adminmode admin_attempts automated_time automated_action syslog_description
	;

adminmode:  ADMIN ADMINMODE 
{
	dprintf(2, "parser: found adminmode \"%s\"", $2);

	if ( lex_state == WITHIN_HOST_DEF ) {

		cur_host->admin_mode = set_admin_mode($2);
		dprintf(3, "parser: cur_host->admin_mode \"%d\"", cur_host->admin_mode); 

	} else if ( lex_state == WITHIN_GLOBAL_DEF ) {
		
		c_test->admin_mode = set_admin_mode($2);
		dprintf(3, "parser: c_test->admin_mode \"%d\"", c_test->admin_mode); 

	}

}
 	| ADMIN ADMINMODE QSTRING
{
	dprintf(2, "parser: found adminmode \"%s\" \"%s\"", $2, $3);
	
	if ( lex_state == WITHIN_HOST_DEF ) {

		cur_host->admin_mode = set_admin_mode($2);
		dprintf(3, "parser: cur_host->admin_mode \"%d\"", cur_host->admin_mode); 
		/*strncat(cur_host->admin_digest, hex2bin_hash($3, digest), SHA1_HASH_SIZE);*/
		hex2bin_hash($3, cur_host->admin_digest);
		/*dprintf(3, "parser: cur_host->admin_digest \"%s\"", cur_host->admin_digest); */
		dprintf(3, "parser: adminhash \"%s\"", $3); 

	} else if ( lex_state == WITHIN_GLOBAL_DEF ) {
		
		c_test->admin_mode = set_admin_mode($2);
		dprintf(3, "parser: c_test->admin_mode \"%d\"", c_test->admin_mode); 
		/*strncat(c_test->admin_digest, $3, SHA1_HASH_SIZE);*/
		hex2bin_hash($3, c_test->admin_digest);
		/*dprintf(3, "parser: c_test->admin_digest \"%s\"", c_test->admin_digest); */
		dprintf(3, "parser: adminhash \"%s\"", $3); 

	}

}

;

admin_attempts: ADMIN_ATTEMPTS DIGITS
{
	dprintf(2, "parser: found admin_attempts \"%d\"", $2);

	if ( lex_state == WITHIN_HOST_DEF ) {
	
		cur_host->max_failures = $2;
		dprintf(3, "parser: cur_host->max_failures \"%d\"", cur_host->max_failures); 

	} else if ( lex_state == WITHIN_GLOBAL_DEF ) {
		
		c_test->max_failures = $2;
		dprintf(3, "parser: c_test->max_failures \"%d\"", c_test->max_failures); 

	}
}
;

logfile: LOGFILE QSTRING
{
	dprintf(2, "parser: logfile: %s",  $2); 
	strncat(c_test->logfile, $2, MAX_LINE_LENGTH);
	dprintf(3, "parser: c_test->logfile \"%s\"", 
			c_test->logfile); 
}
;
	

automated_time: AUTOMATED_TIME a_time_list
	;

automated_action: AUTOMATED_ACTION QSTRING 
{ 
	dprintf(2, "parser: found automated action \"%s\"", $2); 
	strncat(cur_host->automated_action, $2, MAX_LINE_LENGTH);
	dprintf(3, "parser: cur_host->automated_action \"%s\"", 
			cur_host->automated_action); 
}
	;

a_time_list: a_day_spec_unit
 	| a_time_list a_day_spec_unit
	;

a_day_spec_unit: a_day_spec a_time 
{
	dprintf(2, "parser: a day specification unit found");
}
;


a_day_spec: weekday_time_spec
{
	dprintf(2, "parser: periodic time specification");
	cur_time->type = PERIODIC;
	dprintf(3, "parser: cur_time->type \"%d\"", cur_time->type); 
}
  	| YEARLY_DATE_TIME_SPEC
	; 

weekday_time_spec: WEEKDAY 
{ 	
	dprintf(2, "parser: weekday %d specified.", yylval.weekday); 

	if ( lex_state == WITHIN_HOST_DEF ) {
		dprintf(4, "cur_host->automated_time is at %x, c_test->automated_time is at %x.",
				cur_host->automated_time, c_test->host->automated_time);

		cur_time = add_time_front(cur_host->automated_time, new_time());
		dprintf(4, "new tsw_time allocated at %x.", cur_time);

		cur_host->automated_time = cur_time;


		dprintf(4, "cur_host->automated_time is at %x, c_test->automated_time is at %x.", 
				cur_host->automated_time, c_test->host->automated_time);
	} 

	cur_time->weekday[yylval.weekday] = SET;
	dprintf(3, "parser: cur_time->weekday[%d] set (%d).",
			yylval.weekday, cur_time->weekday[yylval.weekday]); 

	dprintf(4, "parser: lex_state \"%d\"", lex_state);
}
	| weekday_time_spec ',' WEEKDAY  
{ 
	dprintf(2, "parser: weekday %d specified.", yylval.weekday ); 
	cur_time->weekday[yylval.weekday] = SET;
	dprintf(3, "parser: cur_time->weekday[%d] set (%d).", 
			yylval.weekday, cur_time->weekday[yylval.weekday]); 
	dprintf(4, "parser: lex_state \"%d\"", lex_state);
}
	| weekday_time_spec '-' WEEKDAY  
{ 
	int active = NO;
	int i;

	dprintf(2, "- weekday %d specified.", yylval.weekday ); 

	for (i = 0; i <= yylval.weekday; i++)
	{
			if ( cur_time->weekday[i] == SET )
			{
				active = SET;
				continue;
			}
			if ( active == SET ) 
			{
				cur_time->weekday[i] = SET;
				dprintf(3, "parser: cur_time->weekday[%d] set (%d).", 
						i, cur_time->weekday[i]); 
			}
	}
	dprintf(4, "parser: lex_state \"%d\"", lex_state);
}
;


a_time: hour_minute '+' time_extension
	;

hour_minute: DIGITS ':' DIGITS
{
	dprintf(2, "parser: hour %d minute %d specified.", $1, $3); 
	cur_time->start_hour = $1;
	cur_time->start_minute = $3;
	dprintf(3, "parser: cur_time->start_hour %d, cur_time->start_minute %d.",
			cur_time->start_hour, cur_time->start_minute);
	dprintf(4, "parser: lex_state \"%d\"", lex_state);
}
;

time_extension: 
	| time_extension seconds
	| time_extension minutes
	| time_extension hours
	| time_extension days
	;


seconds: DIGITS SECOND 
{ 
	dprintf(2, "parser: %d second(s) specified.",  $1); 
	cur_time->cont_second = $1;
	dprintf(3, "parser: cur_time \"%x\"", cur_time);
	dprintf(3, "parser: cur_time->cont_second \"%d\"", cur_time->cont_second);
	dprintf(4, "parser: lex_state \"%d\"", lex_state);
}
;
	

minutes: DIGITS MINUTE 
{ 
	dprintf(2, "parser: %d minute(s) specified.",  $1); 
	cur_time->cont_minute = $1;
	dprintf(3, "parser: cur_time \"%x\"", cur_time);
	dprintf(3, "parser: cur_time->cont_minute \"%d\"", cur_time->cont_minute);
	dprintf(4, "parser: lex_state \"%d\"", lex_state);
}
;

hours: DIGITS HOUR 
{ 
	dprintf(2, "parser: %d hour(s) specified.",  $1); 
	cur_time->cont_hour = $1;
	dprintf(3, "parser: cur_time \"%x\"", cur_time);
	dprintf(3, "parser: cur_time->cont_hour \"%d\"", cur_time->cont_hour);
	dprintf(4, "parser: lex_state \"%d\"", lex_state);
}
;

days: DIGITS DAY 
{ 
	dprintf(2, "parser: %d day(s) specified.",  $1); 
	cur_time->cont_day = $1;
	dprintf(3, "parser: cur_time \"%x\"", cur_time);
	dprintf(3, "parser: cur_time->cont_day \"%d\"", cur_time->cont_day);
	dprintf(4, "parser: lex_state \"%d\"", lex_state);
}
;

syslog_description: SYSLOG syslog_message syslog_facility syslog_priority syslog_tag
	;

syslog_message: SYSLOG_MESSAGE QSTRING
{ 
	dprintf(2, "parser: syslog message \"%s\" specified.",  $2); 
	strncat(cur_host->syslog_message, $2, MAX_LINE_LENGTH);
	dprintf(3, "parser: cur_host->syslog_message \"%s\"", cur_host->syslog_message);
}
;

syslog_facility: SYSLOG_FACILITY QSTRING
{ 
	dprintf(2, "parser: syslog facility \"%s\" specified.",  $2); 
	strncat(cur_host->syslog_facility, $2, MAX_LINE_LENGTH);
	dprintf(3, "parser: cur_host->syslog_facility \"%s\"", cur_host->syslog_facility);
}
;

syslog_priority: SYSLOG_PRIORITY QSTRING
{ 
	dprintf(2, "parser: syslog priority \"%s\" specified.",  $2); 
	strncat(cur_host->syslog_priority, $2, MAX_LINE_LENGTH);
	dprintf(3, "parser: cur_host->syslog_priority \"%s\"", cur_host->syslog_priority);
}
;

syslog_tag: SYSLOG_TAG QSTRING
{ 
	dprintf(2, "parser: syslog tag \"%s\" specified.",  $2); 
	strncat(cur_host->syslog_tag, $2, MAX_LINE_LENGTH);
	dprintf(3, "parser: cur_host->syslog_tag \"%s\"", cur_host->syslog_tag);
}
;

%%

int lineno = 1;

void warning(char *s, char *t) /* print warning message */
{
	fprintf(stderr, "%s: %s", getprogname(), s);
	if (t)
		fprintf(stderr, " %s", t);
	fprintf(stderr, " line %d", lineno);
}

short set_admin_mode(char * mode)
{
	if ( strncasecmp("interactive", mode, MAX_LINE_LENGTH) == 0 ) {
		return INTERACTIVE;
	} else if ( strncasecmp("none", mode, MAX_LINE_LENGTH) == 0 ) {
		return NONE;
	} else if ( strncasecmp("sha1", mode, MAX_LINE_LENGTH) == 0 ) {
		return SHA1;
	} else if ( strncasecmp("global", mode, MAX_LINE_LENGTH) == 0 ) {
		return GLOBAL;
	} else if ( strncasecmp("password", mode, MAX_LINE_LENGTH) == 0 ) {
		return PASSWORD;
	} 
	return NONE;
}


/* no error checking */
int cleanup_hex(char *s1, char *s2)
{
	int i;
	int j=0;

	for (i = 0; i < strlen(s1); i++ ) {
		if ( s1[i] != ' ' )
			s2[j++] = s1[i];
	}
	s2[j] = '\0';
	return 0;
}

/* doesn't check hex validity */
int hex2bin(char h1, char h2) 
{

	int i;
	char s[2];
	int result = 0;
	int factor;

	s[0] = h1;
	s[1] = h2;
	for ( i = 0; i < 2; i++ ) {
		factor = 1;
		if ( i == 0 ) 
			factor = 16;
		switch(s[i]) {
			case '1': result += 1 * factor; break;
			case '2': result += 2 * factor; break;
			case '3': result += 3 * factor; break;
			case '4': result += 4 * factor; break;
			case '5': result += 5 * factor; break;
			case '6': result += 6 * factor; break;
			case '7': result += 7 * factor; break;
			case '8': result += 8 * factor; break;
			case '9': result += 9 * factor; break;
			case 'a': result += 10 * factor; break;
			case 'b': result += 11 * factor; break;
			case 'c': result += 12 * factor; break;
			case 'd': result += 13 * factor; break;
			case 'e': result += 14 * factor; break;
			case 'f': result += 15 * factor; break;
			default: break;
		}
	}		
	dprintf(10,"hex2bin: %s is %d", s, result);
	return result;
}

int hex2bin_hash(char *s, unsigned char *digest)
{
	char h1;
	char h2;
	int i;
	char clean_hex[40];

	dprintf(9,"hex2bin_hash: hex %s", s);
	cleanup_hex(s, clean_hex);
	dprintf(9,"hex2bin_hash: clean hex %s", clean_hex);

	for ( i = 0; i < strlen(clean_hex); i += 2) {
		h1 = clean_hex[i];
		h2 = clean_hex[i+1];
		dprintf(11,"hex: %c%c", h1, h2);
		digest[i/2] = hex2bin(h1, h2);
	}
	return 0;
}

